#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>


//Returns the number of readable regular files in the given directory
//and all subdirectories
//If NULL is provided, it starts at the current working directory
int readable(char* inputPath){
	int CDErr = 0;
	char path[PATH_MAX] = {0};
	if(inputPath != NULL) CDErr = chdir(inputPath);
	if(getcwd(path, PATH_MAX) == NULL) return -errno;
	//Sets errno to EACCES in the case that the first argument is an 
	//unreadable directory
	access(inputPath, R_OK);

	//Attempt to open the path then check for errors from the
	//previous functions
	DIR* dirStream = opendir(path);
	if(dirStream == NULL || CDErr){
		if(errno == ENOTDIR && access(path, R_OK) == 0) return 1;
		else return -errno;
	}

	//Iterate through the directory's contents, recursively exploring
	//subdirectories
	int output = 0;
	struct dirent* entry;
	while((entry = readdir(dirStream)) != NULL){
		//Skip . and .. and get info about the next file
		if(strcmp(entry->d_name, ".")==0 || strcmp(entry->d_name, "..")==0) continue;
		struct stat buf;
		if(lstat(entry->d_name, &buf) != 0) return -errno;

		//Increment the counter if it's a readable regular file,
		//recursive call if it's a reaable directory
		if(S_ISREG(buf.st_mode) && access(entry->d_name, R_OK) == 0) output++;
		if(S_ISDIR(buf.st_mode) && access(entry->d_name, R_OK) == 0){
			int temp = readable(entry->d_name);
			if(chdir("..") != 0) return -errno;
			if(temp > 0) output += temp;
			else if(temp < 0) return -temp;
		}
	}
	if(closedir(dirStream) != 0) return -errno;
	return output;
}